/*
 * Copyright (c) 2009 Anthony Steed and Manuel Fradinho Oliveira
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the 
 * following conditions are met:
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the 
 *    following disclaimer.
 * 2. Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the 
 *    following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of 
 *    this software must display the following acknowledgement:
 *       This product includes software from the book "Networked 
 *       Graphics" written by Anthony Steed and Manuel Fradinho
 *       Oliveira published by Morgan Kaufmann
 * 4. Neither the name of the University nor of the Department
 *    may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 5. Neither the name of Cyntelix may be used to enforese or 
 *    promote products derived from this software without prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS 
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO 
 * EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION). HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact Info: Anthony Steed
 *               a.steed@cs.ucl.ac.uk
 *
 *               Manuel Fradinho Oliveira
 *               mfradinho@cyntelix.com
 *
 */
import java.net.*;
import java.util.*;
import javax.vecmath.*;

public class UDPNetworkServerSimple implements Runnable {
  final int      DEFAULT_NUM_BOIDS = 5;
  DatagramSocket _sendSocket;
  DatagramSocket _receiveSocket;
  int            _sizeBuf;
  byte[]         _buf;
  DatagramPacket _packet;
  Vector         _flocks;
  StringBuffer   _strBuf;
  Thread t = (new Thread(this));


  public UDPNetworkServerSimple (String ip, int local) {
    _flocks = new Vector();
    try {
      _sendSocket = new DatagramSocket();
      _receiveSocket = new DatagramSocket(local,InetAddress.getByName(ip));
      _strBuf = new StringBuffer();
    } catch (Exception e) {
      e.printStackTrace();
      throw new RuntimeException(getClass().getName()+": creating the socket");
    }
  }

  protected void initialise() {
     _sizeBuf = 128 * DEFAULT_NUM_BOIDS;
     _buf = new byte[_sizeBuf];
     _packet = new DatagramPacket(_buf,_buf.length);
     StringBuffer str = new StringBuffer();
  }

  protected StringBuffer prepareAnnounceStr(FlockRemote flock) {
     StringBuffer str = prepareAnnounceStr((Flock)flock);
     str.append(flock.getIPString());
     str.append('#');
     str.append(flock.getPort());
     return str;
  }

  protected StringBuffer prepareAnnounceStr(Flock flock) {
     StringBuffer str = new StringBuffer();
     str.append('!');
     str.append(flock._id);
     str.append("#");
     int red = (int)flock._colour.getX();
     int green = (int)flock._colour.getY();
     int blue = (int)flock._colour.getZ();
     str.append(red);
     str.append("#");
     str.append(green);
     str.append("#");
     str.append(blue);
     str.append("#");
     return str;
  }

  public void receive() {
    String auxStr = null;
    String auxID = null;
    Boid boid = null;
    int index = 0;
    int id = 0;
    FlockRemote auxFlock = null;
    Flock auxFlockRef = null;
    try {
      for (int cycle=0;cycle<_buf.length;cycle++) _buf[cycle] = 0; 
      _receiveSocket.receive(_packet);
      auxStr = new String(_packet.getData());
      index = auxStr.indexOf('!');
      if (index >= 0) {
        auxStr = auxStr.substring(index);
        auxStr = auxStr.substring(index+1);
        index = auxStr.indexOf('#');
        auxID = auxStr.substring(0,index);
        int flockID = Integer.parseInt(auxID);
        auxStr = auxStr.substring(index+1);

        index = auxStr.indexOf('#');
        auxID = auxStr.substring(0,index);
        int red = Integer.parseInt(auxID);
        auxStr = auxStr.substring(index+1);

        index = auxStr.indexOf('#');
        auxID = auxStr.substring(0,index);
        int green = Integer.parseInt(auxID);
        auxStr = auxStr.substring(index+1);

        index = auxStr.indexOf('#');
        auxID = auxStr.substring(0,index);
        int blue = Integer.parseInt(auxID);
        auxStr = auxStr.substring(index+1);

        index = auxStr.indexOf('#');
        String ip = auxStr.substring(0,index);
        auxStr = auxStr.substring(index+1);

        int port = Integer.parseInt(auxStr.trim());

        auxFlock = new FlockRemote(DEFAULT_NUM_BOIDS, new Color3f(red, green, blue));
        auxFlock.setID(flockID);
        auxFlock.setIP(ip);
        auxFlock.setPort(port);

        StringBuffer auxStr2 = null;

        DatagramPacket packet;

        FlockRemote auxflockRemote;
        for (Enumeration e=_flocks.elements();e.hasMoreElements();) {
           auxflockRemote = (FlockRemote)e.nextElement();
           auxStr2 = prepareAnnounceStr(auxflockRemote);
           packet = new DatagramPacket(auxStr2.toString().getBytes(),auxStr2.length());
           packet.setAddress(InetAddress.getByName(ip));
           packet.setPort(port);
           _packet.setPort(auxflockRemote.getPort());
           _sendSocket.send(packet);
           _sendSocket.send(_packet);
        }

        _flocks.add(auxFlock);
      } else {
        index = auxStr.indexOf('#');
        auxID = auxStr.substring(0,index);
        id = Integer.parseInt(auxID);
        FlockRemote auxflockRemote;
        for (Enumeration e=_flocks.elements();e.hasMoreElements();) {
           auxflockRemote = (FlockRemote)e.nextElement();
           if (auxflockRemote._id != id) {
             _packet.setAddress(auxflockRemote.getIP());
             _packet.setPort(auxflockRemote.getPort());
             _sendSocket.send(_packet);
           }
        }
      }

    } catch (Exception e) {
       e.printStackTrace();
    }
  }
  
  public void close() {
    try {  
      _receiveSocket.close();
      _sendSocket.close();
    } catch (Exception e) {
      e.printStackTrace();  
    }
  }

  public void run () {
    initialise();
    while (true) {
      receive();
      Thread.yield();
    }
  }
  
  public void execute() {
    t.start();
  }

  public static void main(String[] args) {
    UDPNetworkServerSimple aux = new UDPNetworkServerSimple("192.168.1.95",5000);
    aux.initialise();
    aux.execute();
  }
}
